//
//  KonohanaDB.m
//  konohana_test
//
//  Created by kunii on 09/11/15.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "KonohanaDB.h"

#define COUNT_PAR_PAGE 5

static NSMutableArray*			contributions = nil;		//	投稿配列

@interface PrivateIDObject : NSObject {
	int			ID;				//	識別子
}
@property (nonatomic, readonly) int		ID;
@end

@implementation PrivateIDObject
@synthesize ID;

/*
	初期化
*/
-(id)init {
	if (self = [super init]) {
		static int index = 1;
		ID = index++;
	}
	return self;
}
@end

/*
	支持クラス
*/
@interface VotePrivate : PrivateIDObject {
	NSString*	comment;		//	支持コメント
}
@property (nonatomic, retain) NSString*	comment;
@end

@implementation VotePrivate
@synthesize comment;

/*
	ダミー情報で初期化
*/
-(id)initWithDummy {
	if (self = [super init]) {
		comment = [[NSString stringWithFormat:@"支持-%d", ID] retain];
	}
	return self;
}

/*
	解放
*/
-(void)dealloc {
	[comment release];
	[super dealloc];
}
@end

/*
	提案クラス
*/
@interface SuggestionPrivate : PrivateIDObject {
	NSString*			name;				//	提案する花の名前
	NSString*			comment;			//	提案名に対するコメント
	int					voteCount;			//	支持総数
	NSMutableArray*		votes;				//	支持配列（コメント付きの支持のみ）
	BOOL				own;				//	この提案が現在の利用者が投稿したものであればYES
}
@property (nonatomic, retain) NSString*			name;
@property (nonatomic, retain) NSString*			comment;
@property (nonatomic, readwrite) int			voteCount;
@property (nonatomic, readonly) NSMutableArray*	votes;
@property (nonatomic, readonly) BOOL			own;
@end

@implementation SuggestionPrivate
@synthesize name;
@synthesize comment;
@synthesize voteCount;
@synthesize votes;
@synthesize own;

/*
	初期化、自分自身を1支持とカウントする。
*/
-(id)init {
	if (self = [super init]) {
		votes = [[NSMutableArray alloc] init];	//	支持配列インスタンス作成
		voteCount = 1;							//	自分自身を1支持とカウントする。
	}
	return self;
}

/*
	ダミー情報で初期化
	ランダムな数のVotePrivateを用意
*/
-(id)initWithDummy {
	if (self = [self init]) {
		name = [[NSString stringWithFormat:@"名前-%d", ID] retain];
		comment = [[NSString stringWithFormat:@"提案-%d", ID] retain];
		if ((rand() % 30) == 0) {				//	1/30の確立で自分の提案とする。
			own = YES;
		}
		int c = rand() % 10;
		for (int i = 0; i < c; i++) {
			[votes insertObject:[[VotePrivate alloc] initWithDummy] atIndex:0];
		}
		voteCount += c + rand() % 5;			//	自分自身による支持数（1） + 支持コメント付き　+　ランダムな支持数
	}
	return self;
}

/*
	解放
*/
-(void)dealloc {
	[name release];
	[comment release];
	[votes release];
	[super dealloc];
}
@end

/*
	問い合わせクラス
*/
@interface ContributionPrivate : PrivateIDObject {
	UIImage*			image;				//	花の写真
	NSString*			name;				//	支持数が一番大きな提案名。初期値は "?"
	NSString*			comment;			//	問い合わせ時のコメント
	NSMutableArray*		suggestions;		//	提案配列
	BOOL				own;				//	この問い合わせが現在の利用者が投稿したものであればYES
}
@property (nonatomic, retain) UIImage*			image;
@property (nonatomic, retain) NSString*			name;
@property (nonatomic, retain) NSString*			comment;
@property (nonatomic, readonly) NSMutableArray*	suggestions;
@property (nonatomic, readonly) BOOL			own;
@end

@implementation ContributionPrivate
@synthesize image;
@synthesize name;
@synthesize comment;
@synthesize own;
@synthesize suggestions;

/*
	初期化、現在の利用者の投稿とする。
*/
-(id)init {
	if (self = [super init]) {
		suggestions = [[NSMutableArray alloc] init];	//	提案配列インスタンス作成
		own = YES;							//	この問い合わせを現在の利用者が投稿したものと設定。
		name = @"?";						//	花の名前の初期値
	}
	return self;
}

/*
	B - Aが負の値になる場合、並べ替えが起こる。
*/
static NSInteger compVoteCount(id A, id B, void* inContext)
{
	return (((SuggestionPrivate*)B).voteCount - ((SuggestionPrivate*)A).voteCount);
}

/*
	支持数の多い提案順に並べ替える
*/
-(void)sort {
	int c = [suggestions count];
	if (c == 0) {
		name = @"?";
		return;
	}
	[suggestions sortUsingFunction:compVoteCount context:nil];
	//	最大支持数の提案名で設定。
	if (c > 0) {
		SuggestionPrivate* instance = [suggestions objectAtIndex:0];
		self.name = instance.name;
	}
}

/*
	ダミー情報で初期化
	ランダムな数のSuggestionPrivateを用意
*/
-(id)initWithDummy {
	if (self = [self init]) {
		comment = [[NSString stringWithFormat:@"問い-%d", ID] retain];
		image = [[UIImage imageNamed:@"dummy.png"] retain];
		//	提案をランダム数用意する。
		int c = rand() % 10;
		for (int i = 0; i < c; i++) {
			[suggestions insertObject:[[SuggestionPrivate alloc] initWithDummy] atIndex:0];
		}
		[self sort];						//	支持数の多い提案順に並べ替える
		own = (rand() % 2) == 1;			//	自分の投稿かどうかをランダムで設定。
	}
	return self;
}

/*
	解放
*/
-(void)dealloc {
	[image release];
	[comment release];
	[suggestions release];
	[super dealloc];
}

/*
	持っている提案のなかに自分の提案があればYESを返す。
*/
-(BOOL)hasOwnSuggestion {
	for (SuggestionPrivate* instance in suggestions) {
		if (instance.own) {
			return YES;
		}
	}
	return NO;
}
@end


@implementation KonohanaDB

@synthesize dbDelegate, filter;

-(void)dummyAction {
	if (rand() % 2) {						//	2回に1回程度はなにもしない。
		return;
	}
	switch (rand() % 6) {
		case 0:	{//	投稿追加
			ContributionPrivate* instance = [[[ContributionPrivate alloc] initWithDummy] autorelease];
			[contributions insertObject:instance atIndex:0];
			break;
		}
		case 1:	{	//	投稿削除
			if ([contributions count] == 0)
				break;
			[contributions removeObjectAtIndex:rand() % [contributions count]];
			break;
		}
		case 2: {	//	提案追加
			if ([contributions count] == 0)
				break;
			ContributionPrivate * contribution = [contributions objectAtIndex:rand() % [contributions count]];
			[contribution.suggestions insertObject:[[SuggestionPrivate alloc] initWithDummy] atIndex:0];
			break;
		}
		case 3: {	//	提案削除
			if ([contributions count] == 0)
				break;
			ContributionPrivate * contribution = [contributions objectAtIndex:rand() % [contributions count]];
			if ([contribution.suggestions count] == 0)
				break;
			[contribution.suggestions removeObjectAtIndex:rand() % [contribution.suggestions count]];
			break;
		}
		case 4: {	//	支持追加
			if ([contributions count] == 0)
				break;
			ContributionPrivate * contribution = [contributions objectAtIndex:rand() % [contributions count]];
			if ([contribution.suggestions count] == 0)
				break;
			SuggestionPrivate* suggestion = [contribution.suggestions objectAtIndex:rand() % [contribution.suggestions count]];
			[suggestion.votes insertObject:[[VotePrivate alloc] initWithDummy] atIndex:0];
			break;
		}
		case 5: {	//	支持削除
			if ([contributions count] == 0)
				break;
			ContributionPrivate * contribution = [contributions objectAtIndex:rand() % [contributions count]];
			if ([contribution.suggestions count] == 0)
				break;
			SuggestionPrivate* suggestion = [contribution.suggestions objectAtIndex:rand() % [contribution.suggestions count]];
			if ([suggestion.votes count] == 0)
				break;
			[suggestion.votes removeObjectAtIndex:rand() % [suggestion.votes count]];
			break;
		}
	}
}

/*
	初期化
*/
-(id)init {
	if (self = [super init]) {
		if (contributions == nil) {
			contributions = [[NSMutableArray alloc] init];	//	投稿配列
			//	10個のランダムな投稿を作成する。
			for (int i = 0; i < 10; i++) {
				ContributionPrivate* instance = [[[ContributionPrivate alloc] initWithDummy] autorelease];
				[contributions insertObject:instance atIndex:0];
			}
		} else {
			[contributions retain];	//	releaseする数と対応を取るため。
		}
	}
	return self;
}

/*
	解放
*/
-(void)dealloc {
	[contributions release];
	[super dealloc];
}

/*
	移譲先に投稿発見を通知。
*/
-(void)handleArrivedContribution:(int)inID photo:(UIImage*)photo name:(NSString*)name comment:(NSString*)comment own:(BOOL)inOwn {
	if ([dbDelegate respondsToSelector:@selector(arrivedContribution:photo:name:comment:own:)]) {
		[dbDelegate arrivedContribution:inID photo:photo name:name comment:comment own:inOwn];
	}
}

/*
	移譲先に提案発見を通知。
*/
-(void)handleArrivedSuggestion:(int)inID name:(NSString*)name comment:(NSString*)comment vote:(int)inVoteCount {
	if ([dbDelegate respondsToSelector:@selector(arrivedSuggestion:name:comment:vote:)]) {
		[dbDelegate arrivedSuggestion:(int)inID name:name comment:comment vote:inVoteCount];
	}
}

/*
	移譲先にコメント付き支持発見を通知。
*/
-(void)handleArrivedVote:(int)inID comment:(NSString*)comment {
	if ([dbDelegate respondsToSelector:@selector(arrivedVote:comment:)]) {
		[dbDelegate arrivedVote:(int)inID comment:comment];
	}
}

/*
	移譲先に終了を通知。
*/
-(void)handleTerminate
{
	if ([dbDelegate respondsToSelector:@selector(terminate)]) {
		[dbDelegate terminate];
	}
}

//	無条件接続
-(void)connect
{
	[self connect:-1 forword:YES];
}

-(void)connect:(int)inID forword:(BOOL)forword {
	[self dummyAction];	
	int count = 0;
	for (ContributionPrivate* instance in contributions) {
		if (forword) {
			if (instance.ID <= inID) {
				break;
			}
		} else {
			if (instance.ID >= inID) {
				continue;
			}
		}
		if (filter == ownContributionOnly) {
			if (!instance.own) {
				continue;
			}
		} else if (filter == ownSuggestionOnly) {
			if (![instance hasOwnSuggestion]) {
				continue;
			}
		}
		[self handleArrivedContribution:instance.ID photo:instance.image name:instance.name comment:instance.comment own:instance.own];
		if (count++ > COUNT_PAR_PAGE)
			break;
	}
	[self handleTerminate];
}

static id findObject(NSArray* array, int inID)
{
	NSEnumerator *enumerator = [array objectEnumerator];
	PrivateIDObject* instance;
	while (instance = [enumerator nextObject]) {
		if (instance.ID == inID)
			return instance;
	}
	return nil;
}

//	投稿IDを指定して検索
-(void)connectForSuggestion:(int)inContributionID {
	[self connectForSuggestion:inContributionID suggestion:-1 forword:YES];
}

//	指定した投稿IDと提案IDより以後または以前（forwordがYESなら以後）に投稿された提案記事を検索
-(void)connectForSuggestion:(int)inContributionID suggestion:(int)inID forword:(BOOL)forword {
	ContributionPrivate* contribution = findObject(contributions, inContributionID);
	int count = 0;
	BOOL find = NO;
	for (SuggestionPrivate* instance in contribution.suggestions) {
		if (forword) {
			if (instance.ID != inID) {
				[self handleArrivedSuggestion:instance.ID name:instance.name comment:instance.comment vote:instance.voteCount];
				if (count++ > COUNT_PAR_PAGE)
					break;
			}
		} else {
			if (!find && (instance.ID != inID)) {
				continue;
			}
			if (!find) {
				find = YES;
				continue;
			}
			[self handleArrivedSuggestion:instance.ID name:instance.name comment:instance.comment vote:instance.voteCount];
			if (count++ > COUNT_PAR_PAGE)
				break;
		}
	}
	[self handleTerminate];
}

//	投稿ID、提案IDを指定して検索
-(void)connectForVote:(int)inContributionID suggestion:(int)inSuggestionID {
	[self connectForVote:inContributionID suggestion:inSuggestionID vote:-1 forword:YES];
}

//	指定した投稿IDと提案ID、指示IDより以後または以前（forwordがYESなら以後）に投稿された指示記事を検索
-(void)connectForVote:(int)inContributionID suggestion:(int)inSuggestionID vote:(int)inID forword:(BOOL)forword {
	ContributionPrivate* contribution = findObject(contributions, inContributionID);
	SuggestionPrivate* suggestion = findObject(contribution.suggestions, inSuggestionID);
	int count = 0;
	for (VotePrivate* instance in suggestion.votes) {
		if (forword) {
			if (instance.ID <= inID) {
				break;
			}
		} else {
			if (instance.ID >= inID) {
				continue;
			}
		}
		[self handleArrivedVote:instance.ID comment:instance.comment];
		if (count++ > COUNT_PAR_PAGE)
			break;
	}
	[self handleTerminate];
}

//	投稿の追加
-(void)addContribution:(UIImage*)inImage comment:(NSString*)comment
{
	ContributionPrivate* instance = [[[ContributionPrivate alloc] init] autorelease];
	instance.image = inImage;
	instance.comment = comment;
	[contributions insertObject:instance atIndex:0];
}

//	指定投稿IDへの提案の追加
-(void)addSuggestion:(int)inContributionID name:(NSString*)inName comment:(NSString*)inComment {
	ContributionPrivate* contribution = findObject(contributions, inContributionID);
	SuggestionPrivate* instance = [[[SuggestionPrivate alloc] init] autorelease];
	instance.name = inName;
	instance.comment = inComment;
	[contribution.suggestions addObject:instance];		//	最後尾に付けるのでソートしない。
}

//	指定投稿ID、提案IDへの支持追加
-(void)addVote:(int)inContributionID suggestionID:(int)inSuggestionID comment:(NSString*)inComment {
	ContributionPrivate* contribution = findObject(contributions, inContributionID);
	SuggestionPrivate* suggestion = findObject(contribution.suggestions, inSuggestionID);
	if (inComment) {
		VotePrivate* instance = [[[VotePrivate alloc] init] autorelease];
		instance.comment = inComment;
		[suggestion.votes insertObject:instance atIndex:0];
	}
	suggestion.voteCount++;
	[contribution sort];						//	支持数の多い提案順に並べ替える
}

//	投稿IDの削除
-(void)removeContribution:(int)inContributionID {
	[contributions removeObject:findObject(contributions, inContributionID)];
}

//	提案IDの削除
-(void)removeSuggestion:(int)inContributionID suggestionID:(int)inSuggestionID {
	ContributionPrivate* contribution = findObject(contributions, inContributionID);
	[contribution.suggestions removeObject:findObject(contribution.suggestions, inSuggestionID)];
}

//	コメント付き支持IDの削除
-(void)removeVote:(int)inContributionID suggestionID:(int)inSuggestionID voteID:(int)inVoteID {
	ContributionPrivate* contribution = findObject(contributions, inContributionID);
	SuggestionPrivate* suggestion = findObject(contribution.suggestions, inSuggestionID);
	suggestion.voteCount--;
	[suggestion.votes removeObject:findObject(suggestion.votes, inVoteID)];
	[contribution sort];						//	支持数の多い提案順に並べ替える
}

@end
