//
//  ImageKit.m
//  NewFlower
//
//  Created by kunii on 10/01/11.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "ImageKit.h"

/*
	グラデーション画像を作成し返す。
*/
static CGImageRef createGradientImage(
	int inWidth,
	int inHeight)
{
	CGImageRef theCGImage = NULL;		//	作成されたグラデーション画像。
	//	グラデーションは白黒の諧調なので色空間はグレーとする。
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceGray();
	//	画像バッファ作成。
	CGContextRef gradientBitmapContext = CGBitmapContextCreate(
		NULL, inWidth, inHeight,
		8, 0, colorSpace, kCGImageAlphaNone);
	if (gradientBitmapContext != NULL) {
		//	バッファが作成できたのでグラデーションをほどこす。
		//		0.0〜1.0のグラデーション。
		//	ただしアルファチャネルは使う使わないに関わらず必須。
		//	そのため、1.0を指定している。
		CGFloat colors[] = {
			0.0, 1.0,			//	白：0.0	α：1.0
			1.0, 1.0};			//	黒：1.0	α：1.0
		//	グラデーション情報の作成。
		CGGradientRef grayScaleGradient
			= CGGradientCreateWithColorComponents(colorSpace, colors, NULL, 2);
		//	このグラデーションを、画像バッファのどの範囲に割り付けるかを設定。
		CGPoint gradientStartPoint = CGPointZero;
		CGPoint gradientEndPoint = CGPointMake(0,inHeight);
		//	描画。
		CGContextDrawLinearGradient(gradientBitmapContext, grayScaleGradient, 
			gradientStartPoint, gradientEndPoint, kCGGradientDrawsAfterEndLocation);
		//	グラデーション情報は用が無くなったので解放。
		CGGradientRelease(grayScaleGradient);
		//	画像バッファをCGImageRefに変換。
		theCGImage = CGBitmapContextCreateImage(gradientBitmapContext);
		//	画像バッファに描画するためのコンテキストを解放。
		CGContextRelease(gradientBitmapContext);
		
	}
	//	色空間情報を解放。
	CGColorSpaceRelease(colorSpace);
	//	グラデーション画像を返す。
    return theCGImage;
}

/*
	inImageを上下反転し、元画像の底からheightピクセル分で切り取った画像を作成し返す。
	画像はアルファ値が1.0から0.0に変化するマスク（アルファチャネル）がついている。
*/
UIImage* createReflectedImage(
	NSUInteger	height, 
	UIImage*	inImage)
{
	//	RGBの描画領域作成。
	CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
	CGContextRef reflectImageContext
		= CGBitmapContextCreate(
		NULL, //	初期化用データ。NULLなら初期化はシステムに任せる
		inImage.size.width,		//	画像横ピクセル数
		height,					//	　　縦
		8,	//	RGB各要素は8ビット
		0,	//	横１ラインの画像を定義するのに必要なバイト数。0はシステムに任せる。
		colorSpace, //	RGB色空間。
		kCGImageAlphaPremultipliedLast);	//	RGBの後ろにアルファ値。
											//	RGBはアルファ値が適用済み。
	//	この時点で色情報は不要なので解放。
    CGColorSpaceRelease(colorSpace);		
	if (reflectImageContext == NULL)	//	作れなかった。
		return NULL;
	//	上下反転画像作成
	CGContextScaleCTM(reflectImageContext, 1, -1);
	CGContextTranslateCTM(reflectImageContext, 0, -(double)height);
	CGContextDrawImage(reflectImageContext, CGRectMake(0, 0, inImage.size.width, inImage.size.height), inImage.CGImage); 
	CGImageRef reflectRGBCGImageRef = CGBitmapContextCreateImage(reflectImageContext);
	CGContextRelease(reflectImageContext);
	//	グラデーションマスク用CGImageRefを作成。
	//	横幅は伸長させるので1ピクセルでいい。
	CGImageRef gradientMaskImageRef = createGradientImage(1, height);
	//	グラデーションマスク付きでCGImageRef作成。
	CGImageRef reflectCGImageRef
		= CGImageCreateWithMask(reflectRGBCGImageRef, gradientMaskImageRef);
	CGImageRelease(reflectRGBCGImageRef);
	CGImageRelease(gradientMaskImageRef);
	// CGImageRefからUIImage作成。
	UIImage *reflectImage = [UIImage imageWithCGImage:reflectCGImageRef];
	//	reflectImageがretainしたので、この関数上は解放する。
	CGImageRelease(reflectCGImageRef);
	return reflectImage;
}

/*
	inImageをinWidth、inHeightにスケーリングした画像を作成し返す。
*/
UIImage* createScaledImage(
	UIImage*	inImage,
	double		inWidth,
	double		inHeight)
{
	CGRect rect = CGRectMake(0.0, 0.0, inWidth, inHeight);
	UIGraphicsBeginImageContext(rect.size);
	//	UIGraphicsGetImageFromCurrentImageContextが呼ばれるまで
	//	描画はすべて新しい描画領域が対象となる。
	[inImage drawInRect:rect];				//	イメージ描画。
	//	新しい描画領域からUIImageを作成。
	UIImage* image = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();			//	解除。
	return image;
}

/*
	inImageをinCropRectにトリミングしたUIImageを返す。
*/
UIImage* createCropedImage(
	UIImage*	inImage,
	CGRect		inCropRect)
{
	UIGraphicsBeginImageContext(inCropRect.size);
	//	UIGraphicsGetImageFromCurrentImageContextが呼ばれるまで
	//	描画はすべて新しい描画領域が対象となる。
	CGPoint origin = {-inCropRect.origin.x, -inCropRect.origin.y};
	[inImage drawAtPoint:origin];				//	イメージ描画。
	//	新しい描画領域からUIImageを作成。
	UIImage* image = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();			//	解除。
	return image;
}
