//
//  konohanaParser.m
//  htmlParser
//
//  Created by kunii on 10/07/06.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "konohanaParser.h"

enum {
	mode_None,
	mode_Row,		//	div class = row,top_suggestion,row_suggestion, row_vote
	mode_Thumb,		//	div class = thumb
	mode_Name,		//	div class = name
	mode_Vote		//	div class = vote
};

//	hrefstrからIDを取り出す。
static int scanContributionID(NSString* hrefstr, NSString* search) {
	NSScanner* scanner = [NSScanner scannerWithString:hrefstr];
	[scanner scanUpToString:search intoString:nil];
	[scanner scanString:search intoString:nil];
	int ID;
	[scanner scanInt:&ID];
	return ID;
}

/*
	投稿解析用クラス
*/
@interface konohanaParserForContribute : konohanaParser {
}
@end

@implementation konohanaParserForContribute
/*
	新しいタグが開いた
*/	
- (void)parser:(NSXMLParser *)parser 
	didStartElement:(NSString *)elementName 
	namespaceURI:(NSString *)namespaceURI 
	qualifiedName:(NSString *)qName 
	attributes:(NSDictionary *)attributeDict {
	NSString* className = [attributeDict valueForKey:@"class"];
	if ([elementName isEqualToString:@"div"]) {
		//	divタグに出会ったので以前のmodeをプッシュ
		[modeStack addObject:[NSNumber numberWithInt:mode]];
		//	divタグに合わせmodeを設定
		if ([className isEqualToString:@"row"]) {
			mode = mode_Row;
		}
		if ([className isEqualToString:@"name"]) {
			mode = mode_Name;
		}
		if ([className isEqualToString:@"thumb"]) {
			mode = mode_Thumb;
		}
	}
	if ((mode == mode_Row) && [elementName isEqualToString:@"a"] && [className isEqualToString:@"contents"]) {
		int ID = scanContributionID([attributeDict valueForKey:@"href"], @"contribution_id=");
		printf("ID = %s [%d]\n", [[attributeDict valueForKey:@"href"] UTF8String], ID);
		NSMutableDictionary* dic = [list objectAtIndex:[list count] - 1];
		[dic setValue:[NSNumber  numberWithInt:ID] forKey:@"ID"];
	}
	if ((mode == mode_Thumb) && [elementName isEqualToString:@"img"]) {
		printf("image URL = %s\n", [[attributeDict valueForKey:@"src"] UTF8String]);
		NSMutableDictionary* dic = [[NSMutableDictionary alloc]init];
		[list addObject:dic];
		[dic setValue:[attributeDict valueForKey:@"src"] forKey:@"url"];
	}
	if ((mode == mode_Thumb) && [elementName isEqualToString:@"a"] && [className isEqualToString:@"delete"]) {
		NSMutableDictionary* dic = [list objectAtIndex:[list count] - 1];
		[dic setValue:[NSNumber numberWithBool:YES] forKey:@"deletable"];
	}
}

/*
	タグが閉じた
*/	
- (void)parser:(NSXMLParser *)parser 
	didEndElement:(NSString *)elementName 
	namespaceURI:(NSString *)namespaceURI 
	qualifiedName:(NSString *)qName {     
	if (mode == mode_Name) {
		NSString* name = [currentParsedCharacterData stringByTrimmingCharactersInSet:[NSCharacterSet  whitespaceAndNewlineCharacterSet]];
		printf("name = %s\n", [name UTF8String]);
		NSMutableDictionary* dic = [list objectAtIndex:[list count] - 1];
		[dic setValue:name forKey:@"name"];
	}
	if ((mode == mode_Row) && [elementName isEqualToString:@"a"]) {
		NSString* comment = [currentParsedCharacterData stringByTrimmingCharactersInSet:[NSCharacterSet  whitespaceAndNewlineCharacterSet]];
		printf("comment = %s\n", [comment UTF8String]);
		NSMutableDictionary* dic = [list objectAtIndex:[list count] - 1];
		[dic setValue:comment forKey:@"comment"];
	}
	if ([elementName isEqualToString:@"div"]) {
		//	divタグが閉じるので、その直前のmodeを復帰させる。
		NSNumber* num = [modeStack objectAtIndex:[modeStack count] - 1];
		mode = [num intValue];
		[modeStack removeLastObject];
	}
	//	<タグ>と</タグ>の間にある文字列はここでリセットする。
	[currentParsedCharacterData release];
	currentParsedCharacterData = nil;
}
@end

/*
	提案解析用クラス
*/
@interface konohanaParserForSuggestion : konohanaParser {
}
@end

@implementation konohanaParserForSuggestion
/*
	新しいタグが開いた
*/	
- (void)parser:(NSXMLParser *)parser 
	didStartElement:(NSString *)elementName 
	namespaceURI:(NSString *)namespaceURI 
	qualifiedName:(NSString *)qName 
	attributes:(NSDictionary *)attributeDict {
	NSString* className = [attributeDict valueForKey:@"class"];
	if ([elementName isEqualToString:@"div"]) {
		//	divタグに出会ったので以前のmodeをプッシュ
		[modeStack addObject:[NSNumber numberWithInt:mode]];
		//	divタグに合わせmodeを設定
		if ([className isEqualToString:@"top_suggestion"] || [className isEqualToString:@"row_suggestion"]) {
			NSMutableDictionary* dic = [[NSMutableDictionary alloc]init];
			[list addObject:dic];
			mode = mode_Row;
		}
		if ([className isEqualToString:@"name"]) {
			mode = mode_Name;
		}
		if ([className isEqualToString:@"vote"]) {
			mode = mode_Vote;
		}
	}
	if ((mode == mode_Row) && ([elementName isEqualToString:@"a"] && [className isEqualToString:@"contents"])) {
		int ID = scanContributionID([attributeDict valueForKey:@"href"], @"suggestion_id=");
		printf("ID = %s [%d]\n", [[attributeDict valueForKey:@"href"] UTF8String], ID);
		NSMutableDictionary* dic = [list objectAtIndex:[list count] - 1];
		[dic setValue:[NSNumber  numberWithInt:ID] forKey:@"ID"];
	}
}

/*
	タグが閉じた
*/	
- (void)parser:(NSXMLParser *)parser 
	didEndElement:(NSString *)elementName 
	namespaceURI:(NSString *)namespaceURI 
	qualifiedName:(NSString *)qName {     
	if (mode == mode_Vote) {
		int vote = scanContributionID(currentParsedCharacterData, @"支持数：");
		printf("vote = %d\n", vote);
		NSMutableDictionary* dic = [list objectAtIndex:[list count] - 1];
		[dic setValue:[NSNumber numberWithInt:vote] forKey:@"vote"];
	}
	if (mode == mode_Name) {
		NSString* name = [currentParsedCharacterData stringByTrimmingCharactersInSet:[NSCharacterSet  whitespaceAndNewlineCharacterSet]];
		printf("name = %s\n", [name UTF8String]);
		NSMutableDictionary* dic = [list objectAtIndex:[list count] - 1];
		[dic setValue:name forKey:@"name"];
	}
	if ([elementName isEqualToString:@"div"]) {
		//	divタグが閉じるので、その直前のmodeを復帰させる。
		NSNumber* num = [modeStack objectAtIndex:[modeStack count] - 1];
		mode = [num intValue];
		[modeStack removeLastObject];
	}
	//	<タグ>と</タグ>の間にある文字列はここでリセットする。
	[currentParsedCharacterData release];
	currentParsedCharacterData = nil;
}
@end

/*
	支持解析用クラス
*/
@interface konohanaParserForVote : konohanaParser {
}
@end

@implementation konohanaParserForVote
/*
	新しいタグが開いた
*/	
- (void)parser:(NSXMLParser *)parser 
	didStartElement:(NSString *)elementName 
	namespaceURI:(NSString *)namespaceURI 
	qualifiedName:(NSString *)qName 
	attributes:(NSDictionary *)attributeDict {
	NSString* className = [attributeDict valueForKey:@"class"];
	if ([elementName isEqualToString:@"div"]) {
		//	divタグに出会ったので以前のmodeをプッシュ
		[modeStack addObject:[NSNumber numberWithInt:mode]];
		//	divタグに合わせmodeを設定
		if ([className isEqualToString:@"row_vote"]) {
			mode = mode_Row;
		}
	}
	if ((mode == mode_Row) && [elementName isEqualToString:@"a"] && [className isEqualToString:@"delete"]) {
		int ID = scanContributionID([attributeDict valueForKey:@"href"], @"delete_id=");
		printf("ID = %s [%d]\n", [[attributeDict valueForKey:@"href"] UTF8String], ID);
		NSString* comment = [currentParsedCharacterData stringByTrimmingCharactersInSet:[NSCharacterSet  whitespaceAndNewlineCharacterSet]];
		printf("comment = %s\n", [comment UTF8String]);
		NSMutableDictionary* dic = [[NSMutableDictionary alloc]init];
		[dic setValue:[NSNumber  numberWithInt:ID] forKey:@"ID"];
		[dic setValue:comment forKey:@"comment"];
		[list addObject:dic];
	}
}

/*
	タグが閉じた
*/	
- (void)parser:(NSXMLParser *)parser 
	didEndElement:(NSString *)elementName 
	namespaceURI:(NSString *)namespaceURI 
	qualifiedName:(NSString *)qName {     
	if ([elementName isEqualToString:@"div"]) {
		//	divタグが閉じるので、その直前のmodeを復帰させる。
		NSNumber* num = [modeStack objectAtIndex:[modeStack count] - 1];
		mode = [num intValue];
		[modeStack removeLastObject];
	}
	//	<タグ>と</タグ>の間にある文字列はここでリセットする。
	[currentParsedCharacterData release];
	currentParsedCharacterData = nil;
}
@end


@implementation konohanaParser
@synthesize list;

-(void)dealloc {
	[modeStack release];
	[list release];
	[super dealloc];
}

-(void)parseStart:(NSXMLParser*)parser {
	parser.delegate = self;
	mode = mode_None;
	[modeStack release];
	modeStack = [[NSMutableArray alloc] init];
	[list release];
	list = [[NSMutableArray alloc] init];
	[parser parse];
}

//	urlで指定されたXMLを解析する。
-(void)parse:(NSString*)url {
	//	URLを指定して直接解析する。
	NSXMLParser* parser = [[NSXMLParser alloc] initWithContentsOfURL:[NSURL URLWithString:url]];
	[self parseStart:parser];
}

//	NSDataで指定されたXMLを解析する。
-(void)parseData:(NSData*)data {
	//	NSDataを指定して解析する。
	NSXMLParser* parser = [[NSXMLParser alloc] initWithData:data];
	[self parseStart:parser];
}

/*
	<タグ>と</タグ>の間にある文字列
*/
- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
    if (!currentParsedCharacterData) {
		currentParsedCharacterData = [[NSMutableString alloc]init];
    }
	[currentParsedCharacterData appendString:string];
}

/*
	エラー発生
*/
- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	NSString* message = [NSString stringWithFormat:@"Error %i, Description: %@, Line: %i, Column: %i", 
		[parseError code], [parseError localizedDescription], [parser lineNumber], [parser columnNumber]];

	printf("error = %s\n", [message UTF8String]);
}

//	typeで指定された解析用クラスを返す
+(konohanaParser*)parserWithType:(int)type {
	if (type == parse_contribute)
		return [[[konohanaParserForContribute alloc] init] autorelease];
	if (type == parse_suggestion)
		return [[[konohanaParserForSuggestion alloc] init] autorelease];
	return [[[konohanaParserForVote alloc] init] autorelease];
}
@end
